package com.youmotic.carpasol.youmotic;

/**
 * Created by Carlos Pascual Olmos on 25/08/2015.
 */

import java.io.File;
import java.io.FileOutputStream;
import org.opencv.android.BaseLoaderCallback;
import org.opencv.android.CameraBridgeViewBase;
import org.opencv.android.CameraBridgeViewBase.CvCameraViewFrame;
import org.opencv.android.CameraBridgeViewBase.CvCameraViewListener2;
import org.opencv.android.LoaderCallbackInterface;
import org.opencv.android.OpenCVLoader;
import org.opencv.android.Utils;
import org.opencv.core.Mat;
import org.opencv.highgui.Highgui;
import org.opencv.imgproc.Imgproc;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.Bitmap.Config;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffXfermode;
import android.hardware.Camera;
import android.media.AudioManager;
import android.media.SoundPool;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.Toast;


public class OpenYouMotic extends Activity implements CvCameraViewListener2 {
    SoundPool soundPool;
    int sonidoclick1;
    int sonidofoto;
    private LayoutInflater myInflater = null;
    private int indiceCamara;
    private static final String STATE_CAMERA_INDEX = "cameraIndex";
    private CameraBridgeViewBase cameraView;
    private boolean guardarSiguienteImagen = false;
    Procesador procesador;
    Bitmap bitmapCara;
    public static String filtro = "Normal";
    public static int color_marco = Color.WHITE;
    public static Button boton_marco_negro;
    public static Button boton_marco_blanco;
    public static Button boton_marco_amarillo;
    public static Button boton_marco_verde;
    public static Button boton_marco_rojo;
    public static Button boton_marco_azul;
    public File[] files;
    public File ruta;
    public int longitudVector;

    private BaseLoaderCallback loaderCallback = new BaseLoaderCallback(this) {
        @Override
        public void onManagerConnected(int status) {
            switch (status) {
                case LoaderCallbackInterface.SUCCESS:
                    cameraView.enableView();
                    break;
                default:
                    super.onManagerConnected(status);
                    break;
            }
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        soundPool = new SoundPool(5, AudioManager.STREAM_MUSIC, 0);

        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        setContentView(R.layout.uno);
        cameraView = (CameraBridgeViewBase) findViewById(R.id.vista_camara);
        cameraView.setCvCameraViewListener(this);

        if (savedInstanceState != null) {
            indiceCamara = savedInstanceState.getInt(
                    STATE_CAMERA_INDEX, 0);
        } else {
            indiceCamara = 0;
        }

        cameraView.setCameraIndex(indiceCamara);

        myInflater = LayoutInflater.from(this);

        View overView = myInflater.inflate(R.layout.unomarco, null);
        this.addContentView(overView, new LayoutParams(
                LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT));

        boton_marco_negro = (Button) findViewById(R.id.btn_marco_negro);
        boton_marco_blanco = (Button) findViewById(R.id.btn_marco_blanco);
        boton_marco_verde = (Button) findViewById(R.id.btn_marco_verde);
        boton_marco_rojo = (Button) findViewById(R.id.btn_marco_rojo);
        boton_marco_azul = (Button) findViewById(R.id.btn_marco_azul);
        boton_marco_amarillo = (Button) findViewById(R.id.btn_marco_amarillo);
        boton_marco_blanco.setAlpha(1);

        ruta = new File(Environment.getExternalStorageDirectory() + "/Android/data/com.youmotic.carpasol.youmotic/images/");
        files = ruta.listFiles();
    }

    @Override
    public void onPause() {
        super.onPause();
        if (cameraView != null)
            cameraView.disableView();
    }

    @Override
    public void onResume() {
        super.onResume();
        sonidoclick1 = soundPool.load(this, R.raw.sonido_click1, 0);
        sonidofoto = soundPool.load(this, R.raw.sonido_foto1, 0);
        OpenCVLoader.initAsync(OpenCVLoader.OPENCV_VERSION_2_4_9, this, loaderCallback);
        files = ruta.listFiles();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (cameraView != null)
            cameraView.disableView();
    }

    public void onCameraViewStarted(int width, int height) {

        procesador = new Procesador();
    }

    public void onCameraViewStopped() {
    }

    public Mat onCameraFrame(CvCameraViewFrame inputFrame) {
        Mat entrada;
        Mat entradacolor;
        entrada = inputFrame.gray();
        entradacolor = inputFrame.rgba();
        Mat salida = procesador.procesa(entrada, entradacolor);

        int maxlineas = salida.rows();
        int maxcolumnas = salida.cols();

        Mat cara = salida.submat(5, maxlineas - 5, maxcolumnas / 8, maxcolumnas / 8 * 7);

        if (guardarSiguienteImagen) {
            takePhoto(cara, cara);
            guardarSiguienteImagen = false;
        }

        if (salida.channels() == 1)
            Imgproc.cvtColor(salida, salida, Imgproc.COLOR_GRAY2RGBA);

        return salida;
    }

    public void onSaveInstanceState(Bundle savedInstanceState) {
        savedInstanceState.putInt(STATE_CAMERA_INDEX, indiceCamara);
        super.onSaveInstanceState(savedInstanceState);
    }

    public void guardarYM(View view) {

        if (files!=null) {
            longitudVector = files.length;
        }
        else{
            longitudVector=0;
        }

        if (longitudVector<21){
        soundPool.play(sonidofoto, 1, 1, 1, 0, 1);
        guardarSiguienteImagen = true;
        CharSequence text = "YEAH, PICTURE SAVED!";
        int duration = Toast.LENGTH_SHORT;
        Toast toast = Toast.makeText(OpenYouMotic.this, text, duration);
        toast.show();}
        else
        {
            Toast.makeText(this, "YOU HAVE MORE THAN 20 YOUMOTICS, PLEASE DELETE AT LEAST ONE OF THEM TO CONTINUE SAVING.THANKS", Toast.LENGTH_LONG).show();
        }
    }

    public void rotarCam(View view) {
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
        indiceCamara++;
        if (indiceCamara == Camera.getNumberOfCameras()) {
            indiceCamara = 0;
        }
        recreate();
    }

    public void volver(View view) {
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
        finish();
    }

    public void filtro1(View view) {
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
        filtro = "Normal";
    }

    public void filtro2(View view) {
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
        filtro = "Filtro1";
    }

    public void filtro3(View view) {
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
        filtro = "Filtro2";
    }

    public void filtro4(View view) {
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
        filtro = "Filtro3";
    }

    public void filtro5(View view) {
        filtro = "Filtro4";
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
    }

    public void filtro6(View view) {
        filtro = "Filtro5";
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
    }

    public void filtro7(View view) {
        filtro = "Filtro6";
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
    }

    public void filtro8(View view) {
        filtro = "Filtro7";
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
    }

    public void filtro9(View view) {
        filtro = "Filtro8";
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
    }

    public void filtro10(View view) {
        filtro = "Filtro9";
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
    }

    public void filtro11(View view) {
        filtro = "Filtro10";
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
    }

    public void filtro12(View view) {
        filtro = "Filtro11";
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
    }

    double d = 0.2;
    float alphainicial = (float) d;

    public void marcoNegro(View view) {
        color_marco = Color.BLACK;
        this.getCurrentFocus();
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
        boton_marco_negro.setAlpha(1);
        boton_marco_blanco.setAlpha(alphainicial);
        boton_marco_rojo.setAlpha(alphainicial);
        boton_marco_azul.setAlpha(alphainicial);
        boton_marco_amarillo.setAlpha(alphainicial);
        boton_marco_verde.setAlpha(alphainicial);
    }

    public void marcoRojo(View view) {
        color_marco = Color.RED;
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
        boton_marco_negro.setAlpha(alphainicial);
        boton_marco_blanco.setAlpha(alphainicial);
        boton_marco_rojo.setAlpha(1);
        boton_marco_azul.setAlpha(alphainicial);
        boton_marco_amarillo.setAlpha(alphainicial);
        boton_marco_verde.setAlpha(alphainicial);
    }

    public void marcoVerde(View view) {
        color_marco = Color.GREEN;
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
        boton_marco_verde.setAlpha(1);
        boton_marco_negro.setAlpha(alphainicial);
        boton_marco_blanco.setAlpha(alphainicial);
        boton_marco_rojo.setAlpha(alphainicial);
        boton_marco_azul.setAlpha(alphainicial);
        boton_marco_amarillo.setAlpha(alphainicial);

    }

    public void marcoAzul(View view) {
        color_marco = Color.BLUE;
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
        boton_marco_azul.setAlpha(1);
        boton_marco_negro.setAlpha(alphainicial);
        boton_marco_blanco.setAlpha(alphainicial);
        boton_marco_rojo.setAlpha(alphainicial);
        boton_marco_amarillo.setAlpha(alphainicial);
        boton_marco_verde.setAlpha(alphainicial);
    }

    public void marcoAmarillo(View view) {
        color_marco = Color.YELLOW;
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
        boton_marco_amarillo.setAlpha(1);
        boton_marco_negro.setAlpha(alphainicial);
        boton_marco_blanco.setAlpha(alphainicial);
        boton_marco_rojo.setAlpha(alphainicial);
        boton_marco_azul.setAlpha(alphainicial);
        boton_marco_verde.setAlpha(alphainicial);
    }

    public void marcoBlanco(View view) {
        color_marco = Color.WHITE;
        soundPool.play(sonidoclick1, 1, 1, 1, 0, 1);
        boton_marco_blanco.setAlpha(1);
        boton_marco_negro.setAlpha(alphainicial);
        boton_marco_rojo.setAlpha(alphainicial);
        boton_marco_azul.setAlpha(alphainicial);
        boton_marco_amarillo.setAlpha(alphainicial);
        boton_marco_verde.setAlpha(alphainicial);
    }

    private void takePhoto(final Mat input, final Mat output) {

        final long currentTimeMillis = System.currentTimeMillis();
        File ruta = new File(Environment.getExternalStorageDirectory() + "/Android/data/com.youmotic.carpasol.youmotic/images/");
        if (!ruta.exists()) {
            ruta.mkdirs();
        }

        final String photoPathOut = ruta + "/YouPhoto_" + currentTimeMillis
                + ".png";

        Mat mBgr = new Mat();
        if (output.channels() == 1)
            Imgproc.cvtColor(output, mBgr, Imgproc.COLOR_GRAY2BGR, 3);
        else
            Imgproc.cvtColor(output, mBgr, Imgproc.COLOR_RGBA2BGR, 3);
        if (!Highgui.imwrite(photoPathOut, mBgr)) {
        }
        if (input.channels() == 1)
            Imgproc.cvtColor(input, mBgr, Imgproc.COLOR_GRAY2BGR, 3);
        else
            Imgproc.cvtColor(input, mBgr, Imgproc.COLOR_RGBA2BGR, 3);
        Imgproc.cvtColor(mBgr, mBgr, Imgproc.COLOR_BGR2RGBA);
        bitmapCara = Bitmap.createBitmap(mBgr.cols(), mBgr.rows(), Config.ARGB_8888);
        Utils.matToBitmap(mBgr, bitmapCara);



        try {

            FileOutputStream out = new FileOutputStream(photoPathOut);
            Bitmap resizedbitmap;
            resizedbitmap = Bitmap.createBitmap(bitmapCara, bitmapCara.getWidth() / 2 - bitmapCara.getHeight() / 2, 0, bitmapCara.getHeight(), bitmapCara.getHeight());
            resizedbitmap = redimensionarImagenMaximo(resizedbitmap, 200, 200);
            resizedbitmap = getRoundedCornerBitmap(resizedbitmap, true);
            resizedbitmap.setHasAlpha(true);
            resizedbitmap.compress(Bitmap.CompressFormat.PNG, 100, out);
            out.flush();
            out.close();

        } catch (Exception error) {
            Log.e("Error saving image", error.getMessage());
        }

        return;

    }

    public static Bitmap getRoundedCornerBitmap(Bitmap input, boolean square) {

        int w = input.getWidth();
        int h = input.getHeight();
        int anchoborde = 20;
        int radius = Math.min(h / 2, w / 2);
        Bitmap output = Bitmap.createBitmap(w + anchoborde * 2, h + anchoborde * 2, Config.ARGB_8888);
        Canvas c = new Canvas(output);
        Paint p = new Paint();
        p.setAntiAlias(true);
        p.setColor(Color.WHITE);
        c.drawARGB(0, 0, 0, 0);
        p.setStyle(Paint.Style.FILL);
        c.drawCircle((w / 2) + anchoborde, (h / 2) + anchoborde, radius, p);
        p.setXfermode(new PorterDuffXfermode(PorterDuff.Mode.SRC_IN));
        c.drawBitmap(input, anchoborde, anchoborde, p);
        p.setXfermode(null);
        p.setAntiAlias(true);
        p.setStyle(Paint.Style.STROKE);
        p.setColor(color_marco);
        p.setStrokeWidth(anchoborde);
        c.drawCircle((w / 2) + anchoborde, (h / 2) + anchoborde, radius, p);
        return output;


    }

    public static Bitmap redimensionarImagenMaximo(Bitmap mBitmap, float newWidth, float newHeigth) {
        int width = mBitmap.getWidth();
        int height = mBitmap.getHeight();
        float scaleWidth = ((float) newWidth) / width;
        float scaleHeight = ((float) newHeigth) / height;
        Matrix matrix = new Matrix();
        matrix.postScale(scaleWidth, scaleHeight);
        return Bitmap.createBitmap(mBitmap, 0, 0, width, height, matrix, false);
    }
}


